-----------------------------------------------------------------------------------------------
-- Client Lua Script for ESS
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- ESS Module Definition
-----------------------------------------------------------------------------------------------
local ESS = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- If you want to modify ESS to swap any other two abilities then just edit the vars below to the relevant IDs and sprites.

-- /eval for k,v in pairs(AbilityBook.GetAbilitiesList()) do if v.strName == "spellname" then Print(v.nId.." "..v.tTiers[1].splObject:GetIcon())end end

local ElectrocuteId = 25473
local BoltCasterId = 20763
local ElectrocuteSprite = "IconSprites:Icon_SkillEngineer_electrocute"
local BoltCasterSprite = "IconSprites:Icon_SkillEngineer_bolt_caster"

local ImpaleId = 23161
local NeutralizeId = 23218
local ImpaleSprite = "IconSprites:Icon_SkillShadow_UI_stlkr_tacticalthrust"
local NeutralizeSprite = "IconSprites:Icon_SkillStalker_Neutralize"

local QuantumCascadeId = 26061
local GammaRaysId = 16322
local QCSprite = "IconSprites:Icon_SkillMedic_quantumcascade"
local GRSprite = "IconSprites:Icon_SkillMedic_gammarays"

local TelekineticStrikeId = 19102
local PsychicFrenzyId = 21613
local TelekineticStrikeSprite = "IconSprites:Icon_SkillMisc_UI_srcr_enhncesns"
local PsychicFrenzySprite = "IconSprites:Icon_SkillEsper_Spectral_Frenzy"

local AtomizeId = 25692
local DualShockId = 47807
local AtomizeSprite = "IconSprites:Icon_SkillMedic_atomize"
local DualShockSprite = "IconSprites:Icon_SkillMedic_paddleshock"

local InactiveSprite = "BK3:UI_BK3_Holo_Framing_2_Blocker"

local ktTexture = {
	{
		name = "White Frame",
		sprite = "CRB_Tooltips:sprTooltip_SquareFrame_White"
	},
	{
		name = "Teal Frame",
		sprite = "CRB_Tooltips:sprTooltip_SquareFrame_UnitTeal"
	},
	{	
		name = "Blue Glow",
		sprite = "AbilitiesSprites:spr_FrameSelection"
	},
	{
		name = "Orange Corners",
		sprite = "CRB_Basekit:kitAccent_Frame_OrangeCorners"
	},
	{
		name = "Bronze Frame", 
		sprite = "BK3:sprMetal_ExpandMenu_Large_Framing"
	},
	{
		name = "Orange Glow",
		sprite = "BK3:UI_BK3_Holo_InsetFramed_Alert"
	},
	{
		name = "Green Pulse",
		sprite = "CRB_DatachronSprites:sprDC_CallPulseAnimation"
	},
	{
		name = "White Pulse",
		sprite ="CRB_PlayerPathSprites:sprPP_ButtonGlow"
	},
	{
		name = "Datachron",
		sprite = "HUD_BottomBar:spr_HUD_DataChron_Base"
	},
}

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ESS:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	if not self.vars then self:SetDefaults() end
    return o
end

function ESS:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- ESS OnLoad
-----------------------------------------------------------------------------------------------
function ESS:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("ESS.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	
end

-----------------------------------------------------------------------------------------------
-- ESS OnDocLoaded
-----------------------------------------------------------------------------------------------
function ESS:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	
	-- Button 1 ------------------------------------------------------------------------
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "Window", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
	    self.wndMain:Show(true, true)
		self.wndMain:SetAnchorOffsets(self.vars.l,self.vars.t,self.vars.r,self.vars.b)
		self:SkillButtonUpdate()
		
	-- Button 2 ------------------------------------------------------------------------
		self.wndMain2 = Apollo.LoadForm(self.xmlDoc, "Window2", nil, self)
		if self.wndMain2 == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
	    self.wndMain2:Show(true, true)
		self.wndMain2:SetAnchorOffsets(self.vars.l2,self.vars.t2,self.vars.r2,self.vars.b2)
		self:SkillButton2Update()

		
		Apollo.RegisterEventHandler("AbilityBookChange","OnAbilityChange",self)
		Apollo.RegisterTimerHandler("ESS_DelayTimer","OnDelayTimer",self)
		Apollo.RegisterSlashCommand("ESS", "OnSlashCommand", self)
		Apollo.RegisterSlashCommand("ess", "OnSlashCommand", self)

	-- Button 1 ------------------------------------------------------------------------		
		self.wndMain:FindChild("Frame"):SetSprite(ktTexture[self.vars.texture].sprite)
		self.wndMain:FindChild("Frame"):SetText(ktTexture[self.vars.texture].name)
		
		self.wndMain:SetStyle("Moveable",self.vars.bLock == 0)
		self.wndMain:SetStyle("Sizable",self.vars.bLock == 0)
	
	-- Button 2 ------------------------------------------------------------------------
		self.wndMain2:FindChild("Frame2"):SetSprite(ktTexture[self.vars.texture].sprite)
		self.wndMain2:FindChild("Frame2"):SetText(ktTexture[self.vars.texture].name)
		
		self.wndMain2:SetStyle("Moveable",self.vars.bLock == 0)
		self.wndMain2:SetStyle("Sizable",self.vars.bLock == 0)
		-- Do additional Addon initialization here
	end
end

-----------------------------------------------------------------------------------------------
-- ESS Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/ESS"
function ESS:OnSlashCommand(strCmd, strArg)
	if strArg == "lock" then
		self.vars.bLock = 1
		self:SetIconLock()
	elseif strArg == "unlock" then
		self.vars.bLock = 0
		self:SetIconLock()		
	elseif strArg == "reset" then
		self:SetDefaults()
	elseif strArg == "swap" then
		self:ToggleESS()
	elseif strArg == "swap2" then
		self:ToggleESS2()
	else
		self:ShowOptionsMenu() -- show the window
	end		
end

function ESS:SetDefaults()
	self.vars = {}
	self.vars.l = 0
	self.vars.t = 0
	self.vars.r = 60
	self.vars.b = 60
	self.vars.l2 = 0
	self.vars.t2 = 0
	self.vars.r2 = 60
	self.vars.b2 = 60
	self.vars.texture = 1
	if self.wndMain then
		self.wndMain:SetAnchorOffsets(self.vars.l,self.vars.t,self.vars.r,self.vars.b)
	end
	if self.wndMain2 then
		self.wndMain2:SetAnchorOffsets(self.vars.l2,self.vars.t2,self.vars.r2,self.vars.b2)
	end
	self.vars.bLock = 0
end

function ESS:ShowOptionsMenu()
	if self.wndOptions then
		self:CloseOptionsMenu()
	end
	self.wndOptions = Apollo.LoadForm(self.xmlDoc,"OptionsMenu",nil,self)
	
	self.wndOptions:FindChild("LockIconButton"):SetCheck(self.vars.bLock == 1)
	self.wndOptions:FindChild("BG:CurrentStyle"):SetSprite(ktTexture[self.vars.texture].sprite)
	self.wndOptions:FindChild("BG:CurrentStyle"):SetText(ktTexture[self.vars.texture].name)
end

function ESS:CloseOptionsMenu()
	if not self.wndOptions then
		return
	end
	self.wndOptions:Destroy()
	self.wndOptions = nil
end

function ESS:OnWindowMouseDown( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation )
	self:ToggleESS()
end

function ESS:OnWindowMouseDownTwo( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation )
	self:ToggleESS2()
end

function ESS:OnAbilityChange()
	self:CreateDelayTimer()
end

function ESS:CreateDelayTimer()
	Apollo.CreateTimer("ESS_DelayTimer",1,false)
end

function ESS:OnDelayTimer()
	self:SkillButtonUpdate()
	self:SkillButton2Update()
end

function ESS:SetIconLock()
	if self.wndMain then
		self.wndMain:SetStyle("Moveable",self.vars.bLock == 0)
		self.wndMain:SetStyle("Sizable",self.vars.bLock == 0)
	end
	if self.wndMain2 then
		self.wndMain2:SetStyle("Moveable",self.vars.bLock == 0)
		self.wndMain2:SetStyle("Sizable",self.vars.bLock == 0)
	end
	if self.wndOptions then
		self.wndOptions:FindChild("LockIconButton"):SetCheck(self.vars.bLock == 1)
	end
end

-- changes the icon of the button
function ESS:SkillButtonUpdate()
	local las = ActionSetLib.GetCurrentActionSet()
	local newsprite,skillsfound
	skillsfound = 0
	if las then
		for k,v in pairs(las) do
			if v == ElectrocuteId then
				newsprite = BoltCasterSprite
				skillsfound = skillsfound + 1
			elseif v == BoltCasterId then
				newsprite = ElectrocuteSprite
				skillsfound = skillsfound + 1
			elseif v == QuantumCascadeId then
				newsprite = GRSprite
				skillsfound = skillsfound + 1
			elseif v == GammaRaysId then
				newsprite = QCSprite
				skillsfound = skillsfound + 1
			elseif v == NeutralizeId then
				newsprite = ImpaleSprite 
				skillsfound = skillsfound + 1
			elseif v == ImpaleId then
				newsprite = NeutralizeSprite 
				skillsfound = skillsfound + 1
			elseif v == TelekineticStrikeId then
				newsprite = PsychicFrenzySprite 
				skillsfound = skillsfound + 1
			elseif v == PsychicFrenzyId then
				newsprite = TelekineticStrikeSprite
				skillsfound = skillsfound + 1
			end
		end
	else
		self:CreateDelayTimer()
	end
	if skillsfound ~= 1 then
		self.wndMain:Show(false,false)
		newsprite = InactiveSprite
	else
		self.wndMain:Show(true,false)
	end
	self.wndMain:FindChild("SkillButton"):SetSprite(newsprite)
end

-- changes the icon of the button
function ESS:SkillButton2Update()
	local las = ActionSetLib.GetCurrentActionSet()
	local newsprite,skillsfound
	skillsfound = 0
	if las then
		for k,v in pairs(las) do
			if v == AtomizeId then
				newsprite = DualShockSprite
				skillsfound = skillsfound + 1
			elseif v == DualShockId then
				newsprite = AtomizeSprite
				skillsfound = skillsfound + 1
			end
		end
	else
		self:CreateDelayTimer()
	end
	if skillsfound ~= 1 then
		self.wndMain2:Show(false,false)
		newsprite = InactiveSprite
	else
		self.wndMain2:Show(true,false)
	end
	self.wndMain2:FindChild("SkillButton2"):SetSprite(newsprite)
end


-- does the switch of the abilities
function ESS:ToggleESS()
	local las = ActionSetLib.GetCurrentActionSet()
	local oldid,newid,oldname,newsprite,skillsfnd,laskey
	skillsfound = 0
	for k,v in pairs(las) do
		if v == ElectrocuteId then
			oldid = ElectrocuteId 
			newid = BoltCasterId
			laskey = k
			newsprite = ElectrocuteSprite
			skillsfound = skillsfound+1
		elseif v == BoltCasterId then
			oldid = BoltCasterId
			newid = ElectrocuteId
			laskey = k
			newsprite = BoltCasterSprite
			skillsfound = skillsfound+1
		elseif v == QuantumCascadeId then
			oldid = QuantumCascadeId
			newid = GammaRaysId
			laskey = k
			newsprite = QCSprite
			skillsfound = skillsfound+1
		elseif v == GammaRaysId then
			oldid = GammaRaysId
			newid = QuantumCascadeId
			laskey = k
			newsprite = GRSprite
			skillsfound = skillsfound+1
		elseif v == ImpaleId then
			oldid = ImpaleId
			newid = NeutralizeId
			laskey = k
			newsprite = ImpaleSprite
			skillsfound = skillsfound+1
		elseif v == NeutralizeId then
			oldid = NeutralizeId
			newid = ImpaleId
			laskey = k
			newsprite = NeutralizeSprite
			skillsfound = skillsfound+1
		elseif v == TelekineticStrikeId then
			oldid = TelekineticStrikeId 
			newid = PsychicFrenzyId 
			laskey = k
			newsprite = TelekineticStrikeSprite 
			skillsfound = skillsfound+1
		elseif v == PsychicFrenzyId then
			oldid = PsychicFrenzyId 
			newid = TelekineticStrikeId 
			laskey = k
			newsprite = PsychicFrenzySprite
			skillsfound = skillsfound+1
		end
	end
	if skillsfound == 1 then -- Only perform swap if ONE of QC or GR found
		local abs = AbilityBook.GetAbilitiesList()
		local nTier
		for k,v in pairs(abs) do
			if v.nId == oldid then
				nTier = v.nCurrentTier
				break
			end
		end
		AbilityBook.UpdateSpellTier(oldid,1)
		AbilityBook.UpdateSpellTier(newid,nTier)
		las[laskey] = newid
		local tResult = ActionSetLib.RequestActionSetChanges(las)
		if tResult.eResult == ActionSetLib.CodeEnumLimitedActionSetResult.Ok then
			self.wndMain:FindChild("SkillButton"):SetSprite(newsprite)
		end
	end
end

function ESS:ToggleESS2()
	local las = ActionSetLib.GetCurrentActionSet()
	local oldid,newid,oldname,newsprite,skillsfnd,laskey
	skillsfound = 0
	for k,v in pairs(las) do
		if v == AtomizeId then
			oldid = AtomizeId
			newid = DualShockId
			laskey = k
			newsprite = AtomizeSprite
			skillsfound = skillsfound+1
		elseif v == DualShockId then
			oldid = DualShockId
			newid = AtomizeId
			laskey = k
			newsprite = DualShockSprite
			skillsfound = skillsfound+1
		end
	end
	if skillsfound == 1 then -- Only perform swap if ONE of QC or GR found
		local abs = AbilityBook.GetAbilitiesList()
		local nTier
		for k,v in pairs(abs) do
			if v.nId == oldid then
				nTier = v.nCurrentTier
				break
			end
		end
		AbilityBook.UpdateSpellTier(oldid,1)
		AbilityBook.UpdateSpellTier(newid,nTier)
		las[laskey] = newid
		local tResult = ActionSetLib.RequestActionSetChanges(las)
		if tResult.eResult == ActionSetLib.CodeEnumLimitedActionSetResult.Ok then
			self.wndMain2:FindChild("SkillButton2"):SetSprite(newsprite)
		end
	end
end

function ESS:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end
	self.vars.l,self.vars.t,self.vars.r,self.vars.b = self.wndMain:GetAnchorOffsets()
	self.vars.l2,self.vars.t2,self.vars.r2,self.vars.b2 = self.wndMain2:GetAnchorOffsets()
	return self.vars
end

function ESS:OnRestore(eLevel, tData)
	if tData then
		local tVarNames = {"l","t","r","b","bLock","texture","l2","t2","r2","b2"}
		for _,strVarName in pairs(tVarNames) do
			if tData[strVarName] then
				self.vars[strVarName] = tData[strVarName]
			end
		end
	end
end


---------------------------------------------------------------------------------------------------
-- OptionsMenu Functions
---------------------------------------------------------------------------------------------------

function ESS:OnButtonCheck( wndHandler, wndControl, eMouseButton )
	if self.vars.bLock == 1 then
		self.vars.bLock = 0
	else
		self.vars.bLock = 1
	end
	self:SetIconLock()
end

function ESS:OnOptionsMenuCloseButton(wndHandler, wndControl, eMouseButton)
	self:CloseOptionsMenu()
end



		---------------------------------------------------------------------------------------------------
		--Frame Style Options!
		---------------------------------------------------------------------------------------------------

function ESS:OnChangeFrameStyleBtnClick( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation )
	self.wndOptions:FindChild("BG:StyleMenu:StyleBtns"):DestroyChildren()
	self:CreateFrameStyles()
	self.wndOptions:FindChild("BG:StyleMenu"):Show(true)
end


function ESS:CreateFrameStyles()
	self.wndFrameStylesList = self.wndOptions:FindChild("BG:StyleMenu:StyleBtns")
	self.tFrameStyles = {}
	local nTextureFrameOffset = 30
	local nYOffset = 0
	local i = 1
	for k,v in pairs(ktTexture) do
		self.tFrameStyles[i] = Apollo.LoadForm(self.xmlDoc, "FrameStyleContainer", self.wndFrameStylesList, self)
		self.tFrameStyles[i]:SetSprite(v.sprite)
		self.tFrameStyles[i]:SetName(v.name)
		self.tFrameStyles[i]:SetText(v.name)
		self.tFrameStyles[i]:SetAnchorPoints(0, 0, 1, 0)
		self.tFrameStyles[i]:SetAnchorOffsets(0, nYOffset, 0, nYOffset + nTextureFrameOffset) 
		nYOffset = nYOffset + nTextureFrameOffset 
		i = i + 1
	end
	local x1, y1, x2, y2 = 166, 57, -28, 123
	self.wndFrameStylesList:GetParent():SetAnchorOffsets(x1, y2, x2, y2 + nYOffset + 10)
	--self.wndConfig:FindChild("Styles"):FindChild("StyleMenu"):SetAnchorOffsets(46, 5, 461, nYOffset+46+25)
end


function ESS:OnNewFrameTextureClicked( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY  )
	self.wndOptions:FindChild("BG:StyleMenu"):Show(false, true)
	local nSelTextureId = 0
	for k,v in pairs(ktTexture) do
		if v.name == wndHandler:GetName() then nSelTextureId = k break end
	end
	self.wndOptions:FindChild("BG:CurrentStyle"):SetSprite(ktTexture[nSelTextureId].sprite)
	self.wndOptions:FindChild("BG:CurrentStyle"):SetText(ktTexture[nSelTextureId].name)
	
	self.wndMain:FindChild("Frame"):SetSprite(ktTexture[nSelTextureId].sprite)
	self.wndMain:FindChild("Frame"):SetText(ktTexture[nSelTextureId].name)
	
	self.wndMain2:FindChild("Frame2"):SetSprite(ktTexture[nSelTextureId].sprite)
	self.wndMain2:FindChild("Frame2"):SetText(ktTexture[nSelTextureId].name)
	
	self.vars.texture = nSelTextureId
end

		---------------------------------------------------------------------------------------------------
		-- Frame Style Options End!
		---------------------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------------
-- ESS Instance
-----------------------------------------------------------------------------------------------
local ESSInst = ESS:new()
ESSInst:Init()
